c  birrp adjustable parameters
c  compile time logical switch, .true. for stand-alone interactive version,
c  .false. for non-interactive use
        logical inter
        parameter (inter=.true.)
c
c  npcsm = maximum number of data pieces
c  nptsm = maximum number of data
c  nptssm = maximum fft section length
c  noutm = maximum number of output time series
c  ninpm = maximum number of input time series
c  nrefm = maximum number of reference time series, must be .ge. ninpm
c  nrsitem = maximum number of reference sites
c  nsectm = maximum number of data sections
c
        parameter (npcsm=2,nptsm=9230000,nptssm=64001)
        parameter (noutm=2,ninpm=2,nrefm=4,nrsitem=2)
        parameter (nsectm=75000)
 
